@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import sys
 
 import pytest
@@ -42,6 +44,7 @@ def writer(tmpdir_cwd):
             index_urls=[],
             trusted_hosts=[],
             format_control=FormatControl(set(), set()),
+            linesep="\n",
             allow_unsafe=False,
             find_links=[],
             emit_find_links=True,
@@ -189,9 +192,9 @@ def test_write_header(writer):
         comment,
         [
             "#",
-            "# This file is autogenerated by pip-compile with python "
+            "# This file is autogenerated by pip-compile with Python "
             f"{sys.version_info.major}.{sys.version_info.minor}",
-            "# To update, run:",
+            "# by the following command:",
             "#",
             "#    pip-compile --output-file={} src_file src_file2".format(
                 writer.click_ctx.params["output_file"].name
@@ -208,9 +211,9 @@ def test_write_header_custom_compile_command(writer, monkeypatch):
         comment,
         [
             "#",
-            "# This file is autogenerated by pip-compile with python "
+            "# This file is autogenerated by pip-compile with Python "
             f"{sys.version_info.major}.{sys.version_info.minor}",
-            "# To update, run:",
+            "# by the following command:",
             "#",
             "#    ./pipcompilewrapper",
             "#",
